import { ConfigProvider } from '@rontrol/config';
import { createPersister } from '@rontrol/persister';
import { AsyncStage, StoreProvider, Store, useStore } from '@react2rx/core';
import { PromptUserConfirmation, ReactorxRouter } from '@react2rx/router';
import { createBrowserHistory, createHashHistory } from 'history';
import { isFunction } from '@rontrol/lodash';
import { StrictMode, useEffect, useMemo } from 'react';
import { createRoot } from 'react-dom/client';
import { useConfirm } from '@rontrol/notify';
import { jsx, jsxs } from 'react/jsx-runtime';

function PersisterConnect(_ref) {
  var {
    persister
  } = _ref;
  var store$ = /*#__PURE__*/useStore();
  useEffect(() => {
    var cleanup = /*#__PURE__*/persister.connect(store$);
    return () => {
      cleanup();
    };
  }, []);
  return null;
}

var HistoryProvider = _ref2 => {
  var {
    children,
    basename = ""
  } = _ref2;
  var confirm = /*#__PURE__*/useConfirm();
  var history = /*#__PURE__*/useMemo(() => createBrowserHistory({}), []);
  return /*#__PURE__*/jsx(PromptUserConfirmation, {
    getUserConfirmation: confirm,
    children: /*#__PURE__*/jsx(ReactorxRouter, {
      basename: basename,
      history: history,
      children: children
    })
  });
};
var HashHistoryProvider = _ref3 => {
  var {
    children
  } = _ref3;
  var confirm = /*#__PURE__*/useConfirm();
  var history = /*#__PURE__*/useMemo(() => createHashHistory({}), []);
  return /*#__PURE__*/jsx(PromptUserConfirmation, {
    getUserConfirmation: confirm,
    children: /*#__PURE__*/jsx(ReactorxRouter, {
      history: history,
      children: children
    })
  });
};
var createBootstrap = config => e => {
  console.log(config);
  var persister = /*#__PURE__*/createPersister({
    name: config.appName || "app"
  });
  return function ($root) {
    var strictMode = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var r = /*#__PURE__*/createRoot($root);
    void persister.hydrate(function () {
      var storeValues = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var store$ = /*#__PURE__*/Store.create(storeValues);

      if (process.env.NODE_ENV !== "production") {
        // @ts-ignore
        void import('redux-logger').then(_ref4 => {
          var {
            createLogger
          } = _ref4;
          store$.applyMiddleware( /*#__PURE__*/createLogger({
            duration: true,
            collapsed: true,
            errorTransformer: e => {
              throw e;
            },
            colors: {
              title: actor => {
                switch (actor.stage) {
                  case AsyncStage.STARTED:
                    return "blue";

                  case AsyncStage.DONE:
                    return "green";

                  case AsyncStage.FAILED:
                    return "red";

                  case AsyncStage.CANCEL:
                    return "orange";
                }

                return "black";
              }
            }
          }));
        });
      }

      var app = /*#__PURE__*/jsx(StoreProvider, {
        value: store$,
        children: /*#__PURE__*/jsxs(ConfigProvider, {
          value: {
            config
          },
          children: [/*#__PURE__*/jsx(PersisterConnect, {
            persister: persister
          }), /*#__PURE__*/jsx(HistoryProvider, {
            basename: config.basename,
            children: isFunction(e) ? e() : e
          })]
        })
      });

      if (strictMode) {
        r.render( /*#__PURE__*/jsx(StrictMode, {
          children: app
        }));
      } else {
        r.render(app);
      }
    });
  };
};

export { HashHistoryProvider, HistoryProvider, createBootstrap };
